#ifndef __TUYA_AI_PROC_H__
#define __TUYA_AI_PROC_H__

#include "tuya_cloud_types.h"
#include "tkl_audio.h"

typedef VOID* TY_AI_PROC_HANDLE;

typedef struct {
    TKL_AUDIO_SAMPLE_E sample_rate;     // audio sample rate
    TKL_AUDIO_DATABITS_E sample_bits;   // audio sample bits
    TKL_AUDIO_CHANNEL_E channel;        // audio channel
    UINT16_T upload_slice_duration;     // 每次上传的音频数据时间片大小，单位ms
    UINT16_T record_duration;           // 最大可缓存的录音时长，单位ms
} TY_AI_PROC_CONFIG_T;

#define TY_AI_PROC_CFG_DEF { \
    .sample_rate = TKL_AUDIO_SAMPLE_16K, \
    .sample_bits = TKL_AUDIO_DATABITS_16, \
    .channel = TKL_AUDIO_CHANNEL_MONO, \
    .upload_slice_duration = 100, \
    .record_duration = 10000, \
}

typedef enum {
    IN_SILENCE,
    IN_START,
    IN_VOICE,
    IN_STOP,
    IN_RESUME,
} TY_AI_VoiceState;

OPERATE_RET ty_ai_proc_create(TY_AI_PROC_HANDLE *handle, TY_AI_PROC_CONFIG_T *cfg);

VOID ty_ai_proc_release(TY_AI_PROC_HANDLE handle);

INT_T ty_ai_audio_stream_write(TY_AI_PROC_HANDLE handle, CHAR_T *buf, UINT_T len);

INT_T ty_ai_audio_stream_read(TY_AI_PROC_HANDLE handle, CHAR_T *buf, UINT_T len);

OPERATE_RET ty_ai_audio_stream_clear(TY_AI_PROC_HANDLE handle);

INT_T ty_ai_audio_stream_get_size(TY_AI_PROC_HANDLE handle);

OPERATE_RET ty_ai_voice_stat_post(TY_AI_PROC_HANDLE handle, TY_AI_VoiceState stat);

OPERATE_RET ty_ai_voice_stat_fetch(TY_AI_PROC_HANDLE handle, TY_AI_VoiceState *stat, UINT_T timeout);

BOOL_T ty_ai_voice_need_keep_session(TY_AI_PROC_HANDLE handle);

CHAR_T *ty_ai_gession_id_get(TY_AI_PROC_HANDLE handle);

OPERATE_RET ty_ai_session_id_set(TY_AI_PROC_HANDLE handle, CHAR_T *session_id);

OPERATE_RET tuya_ai_toy_battery_init(void);

#endif /* __TUYA_AI_PROC_H__ */
