#include "tuya_ai_proc.h"
#include "tuya_ai_mp3_player.h"
#include <stdio.h>
#include "tal_log.h"
#include "tal_memory.h"
#include "tal_thread.h"
#include "tkl_audio.h"
#include "ipc_ringbuf.h"
#include "tuya_devos_utils.h"
#include "tal_semaphore.h"
#include "tal_queue.h"
#include "tal_system.h"
#include "tal_network.h"
#include "tal_mutex.h"
#include "tkl_thread.h"
#include "base_event.h"
#include "ns/audio_subsys_ns.h"
#include "vad/audio_subsys_vad.h"
#include "tuya_voice_protocol.h"
#include "tuya_voice_protocol_upload.h"
#include "speaker_upload.h"
#include "speex_encode.h"
#include "wav_encode.h"
#include "tkl_memory.h"
//#include <driver/audio_ring_buff.h>
// #include "tuya_speaker_service.h"
//#include "tuya_ai_battery.h"

#ifndef AP_ASSERT_CHECK
#define AP_ASSERT_CHECK(EXPR, MSG, ACTION)                                      \
    if (!(EXPR)) {                                                              \
        PR_ERR("ASSERT(%s) has assert failed at %s:%d %s:%s\n",                 \
              #EXPR, __FILE__, __LINE__, __FUNCTION__, MSG);                    \
        ACTION;                                                                 \
    }
#endif /* #ifndef AP_ASSERT_CHECK */

#define PCM_STREAM_BUFF_MAX_LEN

typedef struct {
    TY_AI_PROC_CONFIG_T config;
    BOOL_T is_running;
    TUYA_RINGBUFF_T stream_ringbuf;
    MUTEX_HANDLE ringbuf_mutex;
    QUEUE_HANDLE msg_queue;
    BOOL_T is_empty;
    CHAR_T *curr_session_id;
    BOOL_T curr_is_need_keep;   // 多轮对话使用，多轮对话区分技能
    UINT32_T stream_buf_size;   // 音频流buffer大小，单位字节
    UINT32_T upload_buf_size;   // 分片上传buffer大小，单位字节，默认为100ms
    UCHAR_T *read_buf;
    THREAD_HANDLE task_handle;
} TY_AI_PROC_CONTEXT;

#ifndef TUYA_WS_REQUEST_ID_MAX_LEN
#define TUYA_WS_REQUEST_ID_MAX_LEN       (64)
#endif
STATIC CHAR_T s_tts_request_id[TUYA_WS_REQUEST_ID_MAX_LEN] = {0};
TY_AI_PROC_CONTEXT *s_ctx = NULL;
STATIC VOID _request_id_update(CHAR_T *request_id)
{
    snprintf(s_tts_request_id, TUYA_WS_REQUEST_ID_MAX_LEN, "%s", request_id);
}

STATIC VOID _request_id_reset(VOID)
{
    memset(s_tts_request_id, 0, TUYA_WS_REQUEST_ID_MAX_LEN);
}

STATIC CONST CHAR_T * _get_request_id(VOID)
{
    return s_tts_request_id;
}

STATIC OPERATE_RET _voice_tts_interrupt(void)
{
    PR_NOTICE("voice upload interrupt...");
    if (_get_request_id()[0] == '\0') {
        PR_NOTICE("no request id, ignore");
        return OPRT_OK;
    }
    return tuya_voice_proto_interrupt();
}

STATIC VOID _ai_proc_task(void* arg);

STATIC VOID __tuya_voice_play_tts(TUYA_VOICE_TTS_S *tts)
{
    PR_ERR("%s", __func__);

    /* add play info to playlist for tts */
    AP_ASSERT_CHECK(NULL != tts, "invalid parameter", return;);

    s_ctx->curr_is_need_keep = tts->keep_session;

    /* current session process */
    if (strlen(tts->session_id) > 0 && tts->keep_session) {
        ty_ai_session_id_set(s_ctx, tts->session_id);
    } else {
        ty_ai_session_id_set(s_ctx, NULL);
    }

    // AP_ASSERT_CHECK(OPRT_OK == __catch_playlist_for_tts(tts, TRUE), "catch playlist failed", return;);

    return ;
}

STATIC VOID _tuya_voice_custom(CHAR_T *type, ty_cJSON *json)
{
    PR_ERR("type: %s", type);
    if (json) {
        CHAR_T *data = ty_cJSON_PrintUnformatted(json);
        PR_ERR("json: %s", data);
        ty_cJSON_FreeBuffer(data);
        if (strcmp(type, "response") == 0 && ty_cJSON_GetArraySize(json) == 0) {
            // response为空，表示空语音，播放提示音
            tuya_ai_mp3_player_play_alert(TUYA_AI_ALART_TYPE_PLEASE_AGAIN, TRUE);
        }
    }
    return;
}

STATIC VOID _tuya_voice_stream_player(TUYA_VOICE_STREAM_E type, UCHAR_T *data, INT_T len)
{
    INT_T ret = 0;
    CONST CHAR_T *cur_request_id = NULL;
    cur_request_id = _get_request_id();
    // send data to socket
    switch (type) {
        case TUYA_VOICE_STREAM_START:
            PR_ERR("tts start... requestid=%s", data);
            _request_id_update((CHAR_T *)data);
            tuya_ai_mp3_player_start();
            break;
        case TUYA_VOICE_STREAM_DATA:
            if (!cur_request_id || cur_request_id[0] == '\0') {
                // PR_ERR("tts data, but no request id");
                break;
            }
            PR_ERR("tts data... len=%d, used size=%d", len, tuya_ai_mp3_stream_get_size());
            while (len > 0) {
                ret = tuya_ai_mp3_stream_write((CHAR_T *)data, len);
                if (ret < 0) {
                    PR_ERR("t5_mp3_stream_write failed, ret=%d", ret);
                    break;
                }
                if (ret == 0) {
                    tal_system_sleep(10);
                } else {
                    len -= ret;
                    data += ret;
                }
            }
            if (ret < 0) {
                // stream buffer error, drop all pending data
                _request_id_reset(); // all pending data will be dropped
            }
            break;
        case TUYA_VOICE_STREAM_STOP: {
            if (!cur_request_id || cur_request_id[0] == '\0') {
                PR_ERR("tts stop, but no request id");
                break;
            }
            PR_ERR("tts stop...");
            tuya_ai_mp3_stream_write(NULL, 0);
            _request_id_reset();
            break;
        }
        case TUYA_VOICE_STREAM_ABORT: {
            if (!cur_request_id || cur_request_id[0] == '\0') {
                PR_ERR("tts abort, but no request id");
                break;
            }
            PR_ERR("tts abort...");
            tuya_ai_mp3_player_stop();
            _request_id_reset();
            break;
        }
        default:
            break;
    }
    return;
}

STATIC INT_T _tuya_voice_register_extra_mqt_cb(VOID_T *data)
{
    STATIC BOOL_T registed = FALSE;
    if (registed) {
        return OPRT_OK;
    }
    ;//tuya_voice_proto_start();
    registed = TRUE;
    return OPRT_OK;
}

STATIC INT_T _tuya_voice_register_extra_reset_cb(VOID_T *data)
{
    tuya_voice_proto_del_domain_name();
    return OPRT_OK;
}

STATIC OPERATE_RET _tuya_player_init(VOID)
{
    STATIC BOOL_T player_inited = FALSE;

    PR_ERR("_tuya_player_init start");
    OPERATE_RET ret = OPRT_OK;
    TUYA_VOICE_CBS_S voice_cbc = {0};
    if (player_inited) {
        return OPRT_OK;
    }

    memset(&voice_cbc, 0, sizeof(voice_cbc));
    voice_cbc.tuya_voice_play_tts   = __tuya_voice_play_tts;
    voice_cbc.tuya_voice_custom = _tuya_voice_custom;
    voice_cbc.tuya_voice_tts_stream = _tuya_voice_stream_player;
    ret = tuya_voice_proto_init(&voice_cbc);
    if (ret != OPRT_OK) {
        PR_ERR("tuya_voice_proto_init failed");
        return ret;
    }
    tuya_voice_proto_start();
    //ty_subscribe_event(EVENT_MQTT_CONNECTED, "tts_p", _tuya_voice_register_extra_mqt_cb, SUBSCRIBE_TYPE_NORMAL);
    ty_subscribe_event(EVENT_RESET,"tts_p", _tuya_voice_register_extra_reset_cb, SUBSCRIBE_TYPE_NORMAL);

    PR_ERR("_tuya_player_init end");
    player_inited = TRUE;
    return OPRT_OK;
}

OPERATE_RET ty_ai_proc_create(TY_AI_PROC_HANDLE *handle, TY_AI_PROC_CONFIG_T *cfg)
{
    OPERATE_RET rt = OPRT_OK;
    TY_AI_PROC_CONTEXT *ctx = NULL;

    PR_ERR("ty_ai_proc start...");

    if (s_ctx) {
        PR_ERR("ty_ai_proc already created");
        return OPRT_COM_ERROR;
    }

    // init tuya_voice_protocol
    TUYA_CALL_ERR_GOTO(_tuya_player_init(), error);
    // TUYA_CALL_ERR_RETURN(tuya_speaker_service_init(NULL));
    TUYA_CALL_ERR_GOTO(speaker_intf_encode_register(&global_tuya_speex_encoder), error);
    TUYA_CALL_ERR_GOTO(speaker_intf_encode_register(&global_tuya_wav_encoder), error);

    ctx = tal_malloc(sizeof(TY_AI_PROC_CONTEXT));
    if (ctx == NULL) {
        PR_ERR("malloc failed");
        return OPRT_MALLOC_FAILED;
    }
    memset(ctx, 0, sizeof(TY_AI_PROC_CONTEXT));
    ctx->config = *cfg;
    // calc buffer size
    ctx->upload_buf_size = cfg->upload_slice_duration * cfg->sample_rate * cfg->sample_bits * cfg->channel / 8 / 1000;
    ctx->read_buf = tkl_system_malloc(ctx->upload_buf_size);
    if (ctx->read_buf == NULL) {
        PR_ERR("malloc failed");
        goto error;
    }

    ctx->stream_buf_size = cfg->record_duration * cfg->sample_rate * cfg->sample_bits * cfg->channel / 8 / 1000;
    TUYA_CALL_ERR_GOTO(tuya_ring_buff_create(ctx->stream_buf_size, OVERFLOW_STOP_TYPE, &ctx->stream_ringbuf), error);
    TUYA_CALL_ERR_GOTO(tal_mutex_create_init(&ctx->ringbuf_mutex), error);
    TUYA_CALL_ERR_GOTO(tal_queue_create_init(&ctx->msg_queue, sizeof(INT_T), 8), error);

    ctx->is_running = TRUE;
    TUYA_CALL_ERR_GOTO(tkl_thread_create(&ctx->task_handle, "ai_proc_task", 1024 * 4 * 4, THREAD_PRIO_2, _ai_proc_task, ctx), error);
    PR_ERR("ai_proc_task create success");
    s_ctx = *handle = ctx;

    return OPRT_OK;

error:
    ty_ai_proc_release(ctx);
    return rt;
}

VOID ty_ai_proc_release(TY_AI_PROC_HANDLE handle)
{
    TY_AI_PROC_CONTEXT *ctx = (TY_AI_PROC_CONTEXT *)handle;
    if (ctx == NULL) {
        return;
    }
    // tuya_speaker_service_deinit();

    if (ctx->task_handle) {
        ctx->is_running = FALSE;
    }
    // TODO: wait task exit...

    if (ctx->ringbuf_mutex) {
        tal_mutex_release(ctx->ringbuf_mutex);
    }
    if (ctx->msg_queue) {
        tal_queue_free(ctx->msg_queue);
    }
    if (ctx->stream_ringbuf) {
        tuya_ring_buff_free(ctx->stream_ringbuf);
    }
    if (ctx->read_buf) {
        tkl_system_free(ctx->read_buf);
    }
    tal_free(ctx);
    s_ctx = NULL;
    return;
}

INT_T ty_ai_audio_stream_write(TY_AI_PROC_HANDLE handle, CHAR_T *buf, UINT_T len)
{
    INT_T ret = 0, write_size = 0;
    TY_AI_PROC_CONTEXT *ctx = (TY_AI_PROC_CONTEXT *)handle;
    tal_mutex_lock(ctx->ringbuf_mutex);
    ret = tuya_ring_buff_write(ctx->stream_ringbuf, buf, len);
    tal_mutex_unlock(ctx->ringbuf_mutex);
    if (ret != len) {
        PR_ERR("tuya_ring_buff_write failed, ret=%d, used_size=%d, free_size=%d",
                    ret,
                    tuya_ring_buff_used_size_get(ctx->stream_ringbuf),
                    tuya_ring_buff_free_size_get(ctx->stream_ringbuf));
        return OPRT_COM_ERROR;
    }

    write_size = ret;
    return write_size;
}

INT_T ty_ai_audio_stream_read(TY_AI_PROC_HANDLE handle, CHAR_T *buf, UINT_T len)
{
    INT_T ret = 0, read_size = 0;
    TY_AI_PROC_CONTEXT *ctx = (TY_AI_PROC_CONTEXT *)handle;
    tal_mutex_lock(ctx->ringbuf_mutex);
    ret = tuya_ring_buff_read(ctx->stream_ringbuf, buf, len);
    tal_mutex_unlock(ctx->ringbuf_mutex);
    if (ret < 0) {
        PR_ERR("tuya_ring_buff_read failed, ret=%d", ret);
        return OPRT_COM_ERROR;
    }

    read_size = ret;
    return read_size;
}

OPERATE_RET ty_ai_audio_stream_clear(TY_AI_PROC_HANDLE handle)
{
    OPERATE_RET ret;
    TY_AI_PROC_CONTEXT *ctx = (TY_AI_PROC_CONTEXT *)handle;
    tal_mutex_lock(ctx->ringbuf_mutex);
    ret = tuya_ring_buff_reset(ctx->stream_ringbuf);
    tal_mutex_unlock(ctx->ringbuf_mutex);
    return ret;
}

INT_T ty_ai_audio_stream_get_size(TY_AI_PROC_HANDLE handle)
{
    INT_T size = 0;
    TY_AI_PROC_CONTEXT *ctx = (TY_AI_PROC_CONTEXT *)handle;
    tal_mutex_lock(ctx->ringbuf_mutex);
    size = tuya_ring_buff_used_size_get(ctx->stream_ringbuf);
    tal_mutex_unlock(ctx->ringbuf_mutex);
    return size;
}

OPERATE_RET ty_ai_voice_stat_post(TY_AI_PROC_HANDLE handle, TY_AI_VoiceState stat)
{
    OPERATE_RET ret = OPRT_OK;
    TY_AI_PROC_CONTEXT *ctx = (TY_AI_PROC_CONTEXT *)handle;
    if (ctx == NULL) {
        return OPRT_COM_ERROR;
    }
    INT_T msg = stat;
    ret = tal_queue_post(ctx->msg_queue, &msg, 0);
    return ret;
}

OPERATE_RET ty_ai_voice_stat_fetch(TY_AI_PROC_HANDLE handle, TY_AI_VoiceState *stat, UINT_T timeout)
{
    OPERATE_RET ret = OPRT_OK;
    TY_AI_PROC_CONTEXT *ctx = (TY_AI_PROC_CONTEXT *)handle;
    if (ctx == NULL) {
        return OPRT_COM_ERROR;
    }
    INT_T msg;
    ret = tal_queue_fetch(ctx->msg_queue, &msg, timeout);
    if (ret != OPRT_OK) {
        return ret;
    }
    *stat = msg;
    return OPRT_OK;
}

BOOL_T ty_ai_voice_need_keep_session(TY_AI_PROC_HANDLE handle)
{
    TY_AI_PROC_CONTEXT *ctx = (TY_AI_PROC_CONTEXT *)handle;
    return ctx->curr_is_need_keep;
}

CHAR_T *ty_ai_gession_id_get(TY_AI_PROC_HANDLE handle)
{
    PR_ERR("%s", __func__);
    TY_AI_PROC_CONTEXT *ctx = (TY_AI_PROC_CONTEXT *)handle;
    if (ctx == NULL)
        return NULL;

    PR_ERR("%s: current session id [%s]", __func__, ctx->curr_session_id);
    return ctx->curr_session_id;
}

OPERATE_RET ty_ai_session_id_set(TY_AI_PROC_HANDLE handle, CHAR_T *session_id)
{
    PR_ERR("%s", __func__);
    TY_AI_PROC_CONTEXT *ctx = (TY_AI_PROC_CONTEXT *)handle;
    if (ctx == NULL) {
        return OPRT_COM_ERROR;
    }
    if (ctx->curr_session_id) {
        tal_free(ctx->curr_session_id);
        ctx->curr_session_id = NULL;
    }

    if (session_id) {
        ctx->curr_session_id = tal_malloc(strlen(session_id) + 1);
        memset(ctx->curr_session_id, 0, strlen(session_id) + 1);
        if (ctx->curr_session_id == NULL)
            return OPRT_MALLOC_FAILED;
        strncpy(ctx->curr_session_id, session_id, strlen(session_id));
    }
    PR_ERR("%s: current session id [%s]", __func__, ctx->curr_session_id != NULL ? ctx->curr_session_id : "null");
    return OPRT_OK;
}

STATIC OPERATE_RET _upoad_start(TY_AI_PROC_CONTEXT *ctx)
{
    OPERATE_RET rt = OPRT_OK;
    CHAR_T *session_id = NULL;
    // TODO: ogg-opus instead
    BOOL_T is_region_ay = FALSE;
    // get region info
    CONST CHAR_T *region = get_gw_region();
    if (strcmp(region, "AY") == 0) {
        is_region_ay = TRUE;
    }

    TUYA_VOICE_AUDIO_FORMAT_E format = is_region_ay ? TUYA_VOICE_AUDIO_FORMAT_SPEEX : TUYA_VOICE_AUDIO_FORMAT_WAV;
    SPEAKER_ENCODE_INFO_S param = {
        .encode_type = format,
        .info.channels = 1,
        .info.rate = ctx->config.sample_rate,
        .info.bits_per_sample = ctx->config.sample_bits,
    };
    SPEAKER_UPLOAD_CONFIG_S upload_config = SPEAKER_UPLOAD_CONFIG_FOR_SPEEX();
    if (!is_region_ay) {
        memset(&upload_config, 0, sizeof(upload_config)); // WAV
    }

#if 0
#if 0
    TUYA_VOICE_AUDIO_FORMAT_E format = TUYA_VOICE_AUDIO_FORMAT_SPEEX;
    SPEAKER_ENCODE_INFO_S param;
    memset(&param, 0, sizeof(SPEAKER_ENCODE_INFO_S));
    param.encode_type = format;
    param.info.channels = 1;
    param.info.rate = ctx->config.sample_rate;
    param.info.bits_per_sample = ctx->config.sample_bits;
    SPEAKER_UPLOAD_CONFIG_S upload_config = SPEAKER_UPLOAD_CONFIG_FOR_SPEEX();
#else
    TUYA_VOICE_AUDIO_FORMAT_E format = TUYA_VOICE_AUDIO_FORMAT_WAV;
    SPEAKER_ENCODE_INFO_S param = {
        .encode_type = format,
        .info.channels = 1,
        .info.rate = ctx->config.sample_rate,
        .info.bits_per_sample = ctx->config.sample_bits,
    };
    SPEAKER_UPLOAD_CONFIG_S upload_config = {
        .params = param,
        .report_stat_cb = NULL,
    };
#endif
#endif
    memcpy(&upload_config.params, &param, sizeof(param));
    TUYA_CALL_ERR_RETURN(speaker_intf_upload_init(&upload_config));
    PR_ERR("tuya_voice_upload_init...ok");
    session_id = ty_ai_gession_id_get(ctx);
    if (session_id)
        strncpy(param.session_id, session_id, TUYA_WS_REQUEST_ID_MAX_LEN);
    TUYA_CALL_ERR_RETURN(speaker_intf_upload_media_start(param.session_id));
    PR_ERR("tuya_voice_upload_start...ok");
    if (param.session_id == NULL) {
        TUYA_CALL_ERR_RETURN(speaker_intf_upload_media_get_message_id(param.session_id, TUYA_VOICE_MESSAGE_ID_MAX_LEN));
        PR_ERR("session_id: %s", param.session_id);
    }
    ty_ai_session_id_set(ctx, NULL);

    return OPRT_OK;
}

STATIC OPERATE_RET _upoad_proc(TY_AI_PROC_CONTEXT *ctx, BOOL_T need_force_upload)
{
    OPERATE_RET rt = OPRT_OK;
    INT_T stream_size = 0;
    INT_T read_size = 0;

    stream_size = ty_ai_audio_stream_get_size(ctx);
    PR_ERR("_ai_proc_task stream_size = %d", stream_size);
    if (stream_size < ctx->upload_buf_size && !need_force_upload) {
        return OPRT_OK; // no enough data
    }

    read_size = ty_ai_audio_stream_read(ctx, (CHAR_T *)ctx->read_buf, ctx->upload_buf_size);
    if (read_size < 0) {
        PR_ERR("ty_ai_audio_stream_read failed, ret=%d", read_size);
    }

    // upload data
    PR_ERR("speaker_intf_upload_media_send, len=%d", read_size);
    TUYA_CALL_ERR_RETURN(speaker_intf_upload_media_send(ctx->read_buf, read_size));

    return OPRT_OK;
}

STATIC OPERATE_RET _upload_stop(TY_AI_PROC_CONTEXT *ctx, BOOL_T force_stop)
{
    OPERATE_RET ret = OPRT_OK;
    if (!force_stop) {
        // upload remain data
        while (ty_ai_audio_stream_get_size(ctx) > 0) {
            ret = _upoad_proc(ctx, TRUE);
            if (ret != OPRT_OK) {
                PR_ERR("_upoad_proc failed, ret=%d", ret);
                break;
            }
        }
    }
    // stop upload
    ret = speaker_intf_upload_media_stop(force_stop);
    if (ret != OPRT_OK) {
        PR_ERR("speaker_intf_upload_media_stop failed, ret=%d", ret);
    }
    return ret;
}

STATIC VOID _ai_proc_task(void* arg)
{
    OPERATE_RET ret = OPRT_OK;
    TY_AI_PROC_CONTEXT *ctx = (TY_AI_PROC_CONTEXT *)arg;

    TY_AI_VoiceState stat = IN_SILENCE;
    PR_ERR("ai_proc start...");
    PR_ERR("ctx = %p", ctx);
    while (ctx->is_running) {
        // fetch voice state
        TY_AI_VoiceState cur_stat = stat;
        ret = ty_ai_voice_stat_fetch(ctx, &cur_stat, stat == IN_VOICE ? 30 : 100);
        if (ret != OPRT_OK && stat != IN_VOICE) {
            continue;
        }
        if (ret == OPRT_OK && cur_stat != stat) {
            PR_ERR("stat changed: %d->%d", stat, cur_stat);
            stat = cur_stat;
        }
        switch(stat) {
        case IN_SILENCE:
            PR_ERR("voice silence...");
            _voice_tts_interrupt(); // stop current tts
            _request_id_reset();
            break;
        case IN_START:
            PR_ERR("voice start...");
            // start upload
            ret = _upoad_start(ctx);
            break;
        case IN_VOICE:
            // PR_ERR("voice uploading...");
            // upload data
            ret = _upoad_proc(ctx, FALSE);
            break;
        case IN_STOP:
            PR_ERR("voice stop...");
            // stop upload
            ret = _upload_stop(ctx, FALSE);
            break;
        case IN_RESUME:
            PR_ERR("voice resume...");
            break;
        default:
            break;
        }
    }

    PR_ERR("ai_proc exit...");
}